@@ -1,4 +1,7 @@
+from typing import Optional
+
 from psynet.modular_page import (
+    Control,
     DropdownControl,
     ModularPage,
     NumberControl,
@@ -7,26 +10,29 @@ from psynet.modular_page import (
     TextControl,
 )
 from psynet.timeline import FailedValidation, Module, conditional, join
-from psynet.utils import get_country_dict, get_language_dict, get_logger, get_translator
+from psynet.utils import (
+    get_country_dict,
+    get_language_dict,
+    get_locale,
+    get_logger,
+    get_translator,
+)
 
 logger = get_logger()
 
-DEFAULT_LOCALE = "en"
-
 
 class BasicDemography(Module):
     def __init__(
         self,
-        locale=DEFAULT_LOCALE,
         label="basic_demography",
     ):
         self.label = label
         self.elts = join(
-            Gender(locale=locale),
-            Age(locale=locale),
-            CountryOfBirth(locale=locale),
-            CountryOfResidence(locale=locale),
-            FormalEducation(locale=locale),
+            Gender(),
+            Age(),
+            CountryOfBirth(),
+            CountryOfResidence(),
+            FormalEducation(),
         )
         super().__init__(self.label, self.elts)
 
@@ -35,12 +41,11 @@ class Language(Module):
     def __init__(
         self,
         label="language",
-        locale=DEFAULT_LOCALE,
     ):
         self.label = label
         self.elts = join(
-            MotherTongue(locale=locale),
-            MoreThanOneLanguage(locale=locale),
+            MotherTongue(),
+            MoreThanOneLanguage(),
             conditional(
                 "more_than_one_language",
                 lambda experiment, participant: participant.answer == "yes",
@@ -54,13 +59,12 @@ class BasicMusic(Module):
     def __init__(
         self,
         label="basic_music",
-        locale=DEFAULT_LOCALE,
     ):
         self.label = label
         self.elts = join(
-            YearsOfFormalTraining(locale=locale),
-            HoursOfDailyMusicListening(locale=locale),
-            MoneyFromPlayingMusic(locale=locale),
+            YearsOfFormalTraining(),
+            HoursOfDailyMusicListening(),
+            MoneyFromPlayingMusic(),
         )
         super().__init__(self.label, self.elts)
 
@@ -69,17 +73,16 @@ class Dance(Module):
     def __init__(
         self,
         label="dance",
-        locale=DEFAULT_LOCALE,
     ):
         self.label = label
         self.elts = join(
-            DanceSociallyOrProfessionally(locale=locale),
+            DanceSociallyOrProfessionally(),
             conditional(
                 "dance_socially_or_professionally",
                 lambda experiment, participant: (
                     participant.answer in ["socially", "professionally"]
                 ),
-                LastTimeDanced(locale=locale),
+                LastTimeDanced(),
             ),
         )
         super().__init__(self.label, self.elts)
@@ -89,12 +92,11 @@ class SpeechDisorders(Module):
     def __init__(
         self,
         label="speech_disorders",
-        locale=DEFAULT_LOCALE,
     ):
         self.label = label
         self.elts = join(
-            SpeechLanguageTherapy(locale=locale),
-            DiagnosedWithDyslexia(locale=locale),
+            SpeechLanguageTherapy(),
+            DiagnosedWithDyslexia(),
         )
         super().__init__(self.label, self.elts)
 
@@ -103,11 +105,10 @@ class Income(Module):
     def __init__(
         self,
         label="income",
-        locale=DEFAULT_LOCALE,
     ):
         self.label = label
         self.elts = join(
-            HouseholdIncomePerYear(locale=locale),
+            HouseholdIncomePerYear(),
         )
         super().__init__(self.label, self.elts)
 
@@ -116,13 +117,12 @@ class ExperimentFeedback(Module):
     def __init__(
         self,
         label="feedback",
-        locale=DEFAULT_LOCALE,
     ):
         self.label = label
         self.elts = join(
-            LikedExperiment(locale=locale),
-            FoundExperimentDifficult(locale=locale),
-            EncounteredTechnicalProblems(locale=locale),
+            LikedExperiment(),
+            FoundExperimentDifficult(),
+            EncounteredTechnicalProblems(),
         )
         super().__init__(self.label, self.elts)
 
@@ -132,9 +132,8 @@ class Gender(ModularPage):
     def __init__(
         self,
         label="gender",
-        locale=DEFAULT_LOCALE,
     ):
-        _, _p = get_translator(locale)
+        _p = get_translator(context=True)
         prompt = _p("gender", "How do you identify yourself?")
         self.label = label
         self.prompt = prompt
@@ -152,7 +151,6 @@ class Gender(ModularPage):
             name="gender",
             show_free_text_option=True,
             placeholder_text_free_text=_p("gender", "Specify yourself"),
-            locale=locale,
         )
         super().__init__(
             self.label,
@@ -167,23 +165,21 @@ class Age(ModularPage):
     def __init__(
         self,
         label="age",
-        locale=DEFAULT_LOCALE,
     ):
-        _, _p = get_translator(locale)
+        _p = get_translator(context=True)
         self.label = label
-        self.locale = locale
         self.prompt = _p("age", "What is your age?")
         self.time_estimate = 5
         super().__init__(
             self.label,
             self.prompt,
-            control=NumberControl(locale=locale),
+            control=NumberControl(),
             time_estimate=self.time_estimate,
             save_answer=label,
         )
 
     def validate(self, response, **kwargs):
-        _, _p = get_translator(self.locale)
+        _p = get_translator(context=True)
         answer = response.answer
         error_msg = (
             _p("age", "You need to provide your age as an integer between 0 and 120!")
@@ -201,11 +197,11 @@ class Age(ModularPage):
 
 
 class CountryDropdown(ModularPage):
-    def __init__(self, label, locale):
+    def __init__(self, label):
         self.label = label
-        self.locale = locale
-        _, _p = self.get_translator()
+        _p = get_translator(context=True)
         self.time_estimate = 5
+        locale = get_locale()
         country_dict = get_country_dict(locale)
         control = DropdownControl(
             choices=list(country_dict.keys()) + ["OTHER"],
@@ -213,7 +209,6 @@ class CountryDropdown(ModularPage):
             + [_p("country-select", "Other country")],
             default_text=_p("country-select", "Select a country"),
             name=self.label,
-            locale=locale,
         )
         super().__init__(
             self.label,
@@ -223,14 +218,11 @@ class CountryDropdown(ModularPage):
             save_answer="country",
         )
 
-    def get_translator(self):
-        return get_translator(self.locale)
-
     def get_prompt(self):
         raise NotImplementedError()
 
     def validate(self, response, **kwargs):
-        _, _p = self.get_translator()
+        _p = get_translator(context=True)
         if self.control.force_selection and response.answer == "":
             return FailedValidation(
                 _p("country-select", "You need to select a country!")
@@ -242,12 +234,11 @@ class CountryOfBirth(CountryDropdown):
     def __init__(
         self,
         label="country_of_birth",
-        locale=DEFAULT_LOCALE,
     ):
-        super().__init__(label, locale)
+        super().__init__(label)
 
     def get_prompt(self):
-        _, _p = self.get_translator()
+        _p = get_translator(context=True)
         return _p("country-select", "What country are you from?")
 
 
@@ -255,12 +246,11 @@ class CountryOfResidence(CountryDropdown):
     def __init__(
         self,
         label="country_of_residence",
-        locale=DEFAULT_LOCALE,
     ):
-        super().__init__(label, locale)
+        super().__init__(label)
 
     def get_prompt(self):
-        _, _p = self.get_translator()
+        _p = get_translator(context=True)
         return _p("country-select", "What is your current country of residence?")
 
 
@@ -268,9 +258,8 @@ class FormalEducation(ModularPage):
     def __init__(
         self,
         label="formal_education",
-        locale=DEFAULT_LOCALE,
     ):
-        _, _p = get_translator(locale)
+        _p = get_translator(context=True)
         self.label = label
         self.prompt = _p(
             "formal-education", "What is your highest level of formal education?"
@@ -293,7 +282,6 @@ class FormalEducation(ModularPage):
                 _p("formal-education", "Postgraduate degree or higher"),
             ],
             name="formal_education",
-            locale=locale,
         )
         super().__init__(
             self.label,
@@ -310,17 +298,15 @@ class MotherTongue(ModularPage):
         self,
         label="mother_tongue",
         # TODO Change back to plural (add "(s)") once multi-select is implemented.
-        locale=DEFAULT_LOCALE,
     ):
-        _, _p = get_translator(locale)
+        _p = get_translator(context=True)
         self.label = label
-        self.locale = locale
         self.prompt = _p(
             "language-select",
             "What is your mother tongue - i.e., the language which you have grown up speaking from early childhood?",
         )
         self.time_estimate = 5
-
+        locale = get_locale()
         language_dict = get_language_dict(locale)
 
         control = DropdownControl(
@@ -328,7 +314,6 @@ class MotherTongue(ModularPage):
             labels=list(language_dict.values()) + ["Other language"],
             default_text=_p("language-select", "Select a language"),
             name=self.label,
-            locale=self.locale,
         )
         super().__init__(
             self.label,
@@ -339,7 +324,7 @@ class MotherTongue(ModularPage):
         )
 
     def validate(self, response, **kwargs):
-        _, _p = get_translator(self.locale)
+        _p = get_translator(context=True)
         if self.control.force_selection and response.answer == "":
             return FailedValidation(
                 _p("language-select", "You need to select a language!")
@@ -351,9 +336,9 @@ class MoreThanOneLanguage(ModularPage):
     def __init__(
         self,
         label="more_than_one_language",
-        locale=DEFAULT_LOCALE,
     ):
-        _, _p = get_translator(locale)
+        _ = get_translator()
+        _p = get_translator(context=True)
         self.label = label
         self.prompt = _p("language-select", "Do you speak more than one language?")
         self.time_estimate = 5
@@ -372,45 +357,154 @@ class MoreThanOneLanguage(ModularPage):
         )
 
 
-class LanguagesInOrderOfProficiency(ModularPage):
+class LanguageList(ModularPage):
+    """
+    This is a superclass containing core functionality for free-text participant language input pages.
+
+    Parameters
+    ----------
+    label : str, optional
+        The label for the page. The label is specified with default values in subclasses.
+    prompt : str or callable, optional
+        Prompt to show the participant. The prompt is specified with default values in subclasses.
+    control : Control, optional
+        The type of control to show to the participant. The control is specified with default values in subclasses.
+    time_estimate : float, optional
+        The time estimate for the page.
+    save_answer : str, optional
+        The name under which to store participant answers. Defaults to label.
+
+    Attributes
+    ----------
+    default_label : str
+        A default label to use is none is specified. Defined in each subclass.
+    default_prompt : str or callable
+        A default prompt to use if none is specified. Defined in each subclass.
+    default_control : Control
+        A default control to use if none is specified. Defined in each subclass.
+
+    Methods
+    -------
+    validate(response, **kwargs)
+        Ensures the participant has given a non-empty response, and raises a message to them if not.
+    """
+
     def __init__(
         self,
-        label="languages_in_order_of_proficiency",
-        locale=DEFAULT_LOCALE,
+        label: Optional[str] = None,
+        prompt: Optional[str] = None,
+        control: Optional[Control] = None,
+        time_estimate: float = 5,
+        save_answer: Optional[str] = None,
     ):
-        _, _p = get_translator(locale)
-        self.label = label
-        self.locale = locale
-        self.prompt = _p(
-            "language-select",
-            "Please list the languages you speak in order of proficiency (first language first, second language second, ...)",
-        )
-        self.time_estimate = 5
+        if label is None:
+            label = self.default_label
+        if prompt is None:
+            prompt = self.default_prompt
+        if control is None:
+            control = self.default_control
+        if save_answer is None:
+            save_answer = label
         super().__init__(
-            self.label,
-            self.prompt,
-            control=TextControl(),
-            time_estimate=self.time_estimate,
-            save_answer=label,
+            label,
+            prompt,
+            control=control,
+            time_estimate=time_estimate,
+            save_answer=save_answer,
         )
 
+    @property
+    def default_label(self):
+        raise NotImplementedError("Subclasses must define default_label")
+
+    @property
+    def default_prompt(self):
+        raise NotImplementedError("Subclasses must define default_prompt")
+
+    @property
+    def default_control(self):
+        return TextControl()
+
     def validate(self, response, **kwargs):
-        _, _p = get_translator(self.locale)
-        if not response.answer != "":
+        _p = get_translator(context=True)
+        if response.answer.strip() == "":
             return FailedValidation(
                 _p("language-select", "Please list at least one language!")
             )
         return None
 
 
+class LanguagesInOrderOfProficiency(LanguageList):
+    """
+    A free-text response page for participants to input multiple language in order of proficiency.
+
+    Inherits from
+    -------------
+    LanguageList
+
+    Attributes
+    ----------
+    default_label : str
+        The default label for this page.
+        May be overwritten with a label parameter when called.
+    default_prompt : str or callable
+        The default prompt instructing participants to list languages in order of proficiency.
+        May be overwritten with a prompt parameter when called.
+    default_control : Control
+        The default control used for user input (free-text).
+        May be overwritten with a Control parameter when called (unlikely to be needed).
+    """
+
+    default_label = "languages_in_order_of_proficiency"
+
+    @property
+    def default_prompt(self):
+        _p = get_translator(context=True)
+        return _p(
+            "language-select",
+            "Please list the languages you speak in order of proficiency (first language first, second language second, ...)",
+        )
+
+
+class MotherTongues(LanguageList):
+    """
+    A free-text response page for participants to input multiple languages they speak as mother tongues.
+
+    Inherits from
+    -------------
+    LanguageList
+
+    Attributes
+    ----------
+    default_label : str
+        The default label for this page.
+        May be overwritten with a label parameter when called.
+    default_prompt : str or callable
+        The default prompt instructing participants to list all languages they speak as mother tongues.
+        May be overwritten with a prompt parameter when called.
+    default_control : Control
+        The default control used for user input (free-text).
+        May be overwritten with a Control parameter when called (unlikely to be needed).
+    """
+
+    default_label = "mother_tongues"
+
+    @property
+    def default_prompt(self):
+        _p = get_translator(context=True)
+        return _p(
+            "language-select",
+            "Please list all languages you speak as mother tongues (i.e., which you grew up speaking since childhood).",
+        )
+
+
 # Basic music #
 class YearsOfFormalTraining(ModularPage):
     def __init__(
         self,
         label="years_of_formal_training",
-        locale=DEFAULT_LOCALE,
     ):
-        _, _p = get_translator(locale)
+        _p = get_translator(context=True)
         self.label = label
         self.prompt = _p(
             "music",
@@ -420,7 +514,7 @@ class YearsOfFormalTraining(ModularPage):
         super().__init__(
             self.label,
             self.prompt,
-            control=NumberControl(locale=locale),
+            control=NumberControl(),
             time_estimate=self.time_estimate,
             save_answer=label,
         )
@@ -430,9 +524,8 @@ class HoursOfDailyMusicListening(ModularPage):
     def __init__(
         self,
         label="hours_of_daily_music_listening",
-        locale=DEFAULT_LOCALE,
     ):
-        _, _p = get_translator(locale)
+        _p = get_translator(context=True)
         self.label = label
         self.prompt = _p(
             "music", "On average, how many hours do you listen to music daily?"
@@ -441,7 +534,7 @@ class HoursOfDailyMusicListening(ModularPage):
         super().__init__(
             self.label,
             self.prompt,
-            control=NumberControl(locale=locale),
+            control=NumberControl(),
             time_estimate=self.time_estimate,
             save_answer=label,
         )
@@ -451,9 +544,8 @@ class MoneyFromPlayingMusic(ModularPage):
     def __init__(
         self,
         label="money_from_playing_music",
-        locale=DEFAULT_LOCALE,
     ):
-        _, _p = get_translator(locale)
+        _p = get_translator(context=True)
         self.label = label
         self.prompt = _p("music", "Do you make money from playing music?")
         self.time_estimate = 5
@@ -462,7 +554,6 @@ class MoneyFromPlayingMusic(ModularPage):
             ["frequently", "sometimes", "never"],
             [_p("music", "Frequently"), _p("music", "Sometimes"), _p("music", "Never")],
             name="money_from_playing_music",
-            locale=locale,
         )
         super().__init__(
             self.label,
@@ -478,9 +569,9 @@ class HearingLoss(ModularPage):
     def __init__(
         self,
         label="hearing_loss",
-        locale=DEFAULT_LOCALE,
     ):
-        _, _p = get_translator(locale)
+        _ = get_translator()
+        _p = get_translator(context=True)
         self.label = label
         self.prompt = _p(
             "music", "Do you have hearing loss or any other hearing issues?"
@@ -506,9 +597,8 @@ class DanceSociallyOrProfessionally(ModularPage):
     def __init__(
         self,
         label="dance_socially_or_professionally",
-        locale=DEFAULT_LOCALE,
     ):
-        _, _p = get_translator(locale)
+        _p = get_translator(context=True)
         self.label = label
         self.prompt = _p("dance", "Do you dance socially or professionally?")
         self.time_estimate = 5
@@ -521,7 +611,6 @@ class DanceSociallyOrProfessionally(ModularPage):
                 _p("dance", "I never dance"),
             ],
             name="dance_socially_or_professionally",
-            locale=locale,
         )
         super().__init__(
             self.label,
@@ -536,9 +625,8 @@ class LastTimeDanced(ModularPage):
     def __init__(
         self,
         label="last_time_danced",
-        locale=DEFAULT_LOCALE,
     ):
-        _, _p = get_translator(locale)
+        _p = get_translator(context=True)
         self.label = label
         self.prompt = _p(
             "dance",
@@ -564,7 +652,6 @@ class LastTimeDanced(ModularPage):
                 _p("dance", "I never danced"),
             ],
             name="last_time_danced",
-            locale=locale,
         )
         super().__init__(
             self.label,
@@ -580,9 +667,9 @@ class SpeechLanguageTherapy(ModularPage):
     def __init__(
         self,
         label="speech_language_therapy",
-        locale=DEFAULT_LOCALE,
     ):
-        _, _p = get_translator(locale)
+        _ = get_translator()
+        _p = get_translator(context=True)
         self.label = label
         self.prompt = _p(
             "speech-disorder", "Did you get speech-language therapy as a child?"
@@ -607,9 +694,9 @@ class DiagnosedWithDyslexia(ModularPage):
     def __init__(
         self,
         label="diagnosed_with_dyslexia",
-        locale=DEFAULT_LOCALE,
     ):
-        _, _p = get_translator(locale)
+        _ = get_translator()
+        _p = get_translator(context=True)
         self.label = label
         self.prompt = _p(
             "speech-disorder", "Have you ever been diagnosed with dyslexia?"
@@ -636,9 +723,8 @@ class HouseholdIncomePerYear(ModularPage):
         self,
         label="household_income_per_year",
         currency="USD",
-        locale=DEFAULT_LOCALE,
     ):
-        _, _p = get_translator(locale)
+        _p = get_translator(context=True)
         self.label = label
         self.prompt = _p("income", "What is your total household income per year?")
         self.time_estimate = 5
@@ -673,7 +759,6 @@ class HouseholdIncomePerYear(ModularPage):
                 _p("income", "150,000 {CURRENCY} or more").format(CURRENCY=currency),
             ],
             name="household_income_per_year",
-            locale=locale,
         )
         super().__init__(
             self.label,
@@ -689,9 +774,8 @@ class LikedExperiment(ModularPage):
     def __init__(
         self,
         label="liked_experiment",
-        locale=DEFAULT_LOCALE,
     ):
-        _, _p = get_translator(locale)
+        _p = get_translator(context=True)
         self.label = label
         self.prompt = _p("experiment-feedback", "Did you like the experiment?")
         self.time_estimate = 5
@@ -710,9 +794,8 @@ class FoundExperimentDifficult(ModularPage):
     def __init__(
         self,
         label="find_experiment_difficult",
-        locale=DEFAULT_LOCALE,
     ):
-        _, _p = get_translator(locale)
+        _p = get_translator(context=True)
         self.label = label
         self.prompt = _p(
             "experiment-feedback", "Did you find the experiment difficult?"
@@ -732,9 +815,8 @@ class EncounteredTechnicalProblems(ModularPage):
     def __init__(
         self,
         label="encountered_technical_problems",
-        locale=DEFAULT_LOCALE,
     ):
-        _, _p = get_translator(locale)
+        _p = get_translator(context=True)
         self.label = label
         self.prompt = (
             _p(
