@@ -2,10 +2,10 @@
 
 import json
 from smtplib import SMTPAuthenticationError
+from typing import TYPE_CHECKING, Dict
 
 import dallinger.models
 from dallinger import db
-from dallinger.config import get_config
 from dallinger.notifications import admin_notifier
 from sqlalchemy import (
     Boolean,
@@ -24,10 +24,14 @@ from sqlalchemy.orm.collections import attribute_mapped_collection
 from .asset import AssetParticipant
 from .data import SQLMixinDallinger
 from .field import PythonList, PythonObject, VarStore, extra_var
-from .utils import get_logger, organize_by_key
+from .utils import call_function_with_context, get_config, get_logger, organize_by_key
 
 logger = get_logger()
 
+if TYPE_CHECKING:
+    from .sync import SyncGroup
+    from .timeline import Module
+
 # pylint: disable=unused-import
 
 UniqueConstraint(dallinger.models.Participant.worker_id)
@@ -154,11 +158,26 @@ class Participant(SQLMixinDallinger, dallinger.models.Participant):
 
     elt_id = Column(PythonList)
     elt_id_max = Column(PythonList)
+
+    time_credit = Column(Float)
+    estimated_max_time_credit = Column(Float)
+    progress = Column(Float)
+
+    # If time_credit_fixes is non-empty, then the last element of time_credit_fixes
+    # is used as the currently active time credit 'fix'. While this fix is active,
+    # the participant's time credit will not be allowed to increase above this number.
+    # Once the fix expires, the participant's time credit will be set to that number exactly.
+    # See ``psynet.timeline.with_fixed_time_credit`` for an explanation.
+    time_credit_fixes = Column(PythonList)
+    # progress_fixes is analogous to time_credit_fixes, but for progress.
+    progress_fixes = Column(PythonList)
+
     page_uuid = Column(String)
     aborted = Column(Boolean, default=False)
     complete = Column(Boolean, default=False)
     answer = Column(PythonObject)
     answer_accumulators = Column(PythonList)
+    sequences = Column(PythonList)
     branch_log = Column(PythonObject)
     for_loops = Column(PythonObject, default=lambda: {})
     failure_tags = Column(PythonList, default=lambda: [])
@@ -182,6 +201,11 @@ class Participant(SQLMixinDallinger, dallinger.models.Participant):
 
     all_responses = relationship("psynet.timeline.Response")
 
+    awaited_async_code_block_process_id = Column(Integer, ForeignKey("process.id"))
+    awaited_async_code_block_process = relationship(
+        "AsyncProcess", foreign_keys=[awaited_async_code_block_process_id]
+    )
+
     # @property
     # def current_trial(self):
     #     if self.in_module and hasattr(self.module_state, "current_trial"):
@@ -270,11 +294,11 @@ class Participant(SQLMixinDallinger, dallinger.models.Participant):
     # sync_groups is a relationship that gives a list of all SyncGroups for that participnat
 
     @property
-    def active_sync_groups(self):
+    def active_sync_groups(self) -> Dict[str, "SyncGroup"]:
         return {group.group_type: group for group in self.sync_groups if group.active}
 
     @property
-    def sync_group(self):
+    def sync_group(self) -> "SyncGroup":
         candidates = self.active_sync_groups
         if len(candidates) == 1:
             return list(candidates.values())[0]
@@ -375,11 +399,23 @@ class Participant(SQLMixinDallinger, dallinger.models.Participant):
             if log.finished
         ]
 
-    def start_module(self, module):
+    def start_module(self, module: "Module"):
+        self.check_module_not_already_started(module)
         state = module.state_class(module, self)
         state.start()
         self.module_state = state
 
+    def check_module_not_already_started(self, module: "Module"):
+        if module.id not in self.module_states:
+            return
+        else:
+            states = self.module_states[module.id]
+            for state in states:
+                if not state.finished:
+                    raise RuntimeError(
+                        f"Participant already has an unfinished module state for '{module.id}'..."
+                    )
+
     def end_module(self, module):
         # This should only fail (delivering multiple logs) if the experimenter has perversely
         # defined a recursive module (or is reusing module ID)
@@ -393,7 +429,10 @@ class Participant(SQLMixinDallinger, dallinger.models.Participant):
             )
         elif len(state) > 1:
             raise RuntimeError(
-                f"Participant had multiple unfinished module states with id = '{module.id}'."
+                (
+                    f"Participant had multiple unfinished module states with id = '{module.id}': "
+                    f"{[s.__json__() for s in state]}, participant: {self.__json__()}"
+                )
             )
 
         state = state[0]
@@ -428,11 +467,18 @@ class Participant(SQLMixinDallinger, dallinger.models.Participant):
     def __init__(self, experiment, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.vars = {}
+        self.time_credit = 0.0
+        self.estimated_max_time_credit = (
+            experiment.timeline.estimated_time_credit.get_max("time")
+        )
+        self.progress = 0.0
+        self.time_credit_fixes = []
+        self.progress_fixes = []
         self.elt_id = [-1]
         self.elt_id_max = [len(experiment.timeline) - 1]
         self.answer_accumulators = []
+        self.sequences = []
         self.complete = False
-        self.time_credit.initialize(experiment)
         self.performance_reward = 0.0
         self.unpaid_bonus = 0.0
         self.base_payment = experiment.base_payment
@@ -449,6 +495,29 @@ class Participant(SQLMixinDallinger, dallinger.models.Participant):
     def initialize(self, experiment):
         pass
 
+    @property
+    def locale(self):
+        return self.var.get("locale", default=None)
+
+    @property
+    def failure_cascade(self):
+        return [lambda: self.alive_trials]
+
+    @property
+    def gettext(self):
+        return self.translator[0]
+
+    @property
+    def pgettext(self):
+        return self.translator[1]
+
+    @property
+    def time_reward(self):
+        wage_per_hour = get_config().get("wage_per_hour")
+        seconds = self.time_credit
+        hours = seconds / 3600
+        return hours * wage_per_hour
+
     def calculate_reward(self):
         """
         Calculates and returns the currently accumulated reward for the given participant.
@@ -457,10 +526,23 @@ class Participant(SQLMixinDallinger, dallinger.models.Participant):
             The reward as a ``float``.
         """
         return round(
-            self.time_credit.get_time_reward() + self.performance_reward,
+            self.time_reward + self.performance_reward,
             ndigits=2,
         )
 
+    def inc_time_credit(self, time_credit: float):
+        new_value = self.time_credit + time_credit
+        new_value = min([new_value, *self.time_credit_fixes])
+        self.time_credit = new_value
+
+    def inc_progress(self, time_credit: float):
+        if self.estimated_max_time_credit == 0.0:
+            new_value = 1.0
+        else:
+            new_value = self.progress + time_credit / self.estimated_max_time_credit
+            new_value = min([new_value, *self.progress_fixes])
+        self.progress = new_value
+
     def inc_performance_reward(self, value):
         self.performance_reward += value
 
@@ -523,15 +605,6 @@ class Participant(SQLMixinDallinger, dallinger.models.Participant):
             .first()
         )
 
-    @property
-    @extra_var(__extra_vars__)
-    def progress(self):
-        return 1.0 if self.complete else self.time_credit.progress
-
-    @property
-    def time_credit(self):
-        return TimeCreditStore(self)
-
     def append_branch_log(self, entry: str):
         # We need to create a new list otherwise the change may not be recognized
         # by SQLAlchemy(?)
@@ -576,17 +649,6 @@ class Participant(SQLMixinDallinger, dallinger.models.Participant):
         self.failure_tags = combined
         return self
 
-    def get_locale(self, experiment):
-        if self.var.has("locale"):
-            return self.var.locale
-        else:
-            locale = experiment.var.current_locale
-            self.var.set("locale", locale)
-            logger.warning(
-                f"Participant {self.id} locale was not set, setting to default locale of the experiment: {locale}"
-            )
-            return locale
-
     def abort_info(self):
         """
             Information that will be shown to a participant if they click the abort button,
@@ -601,6 +663,45 @@ class Participant(SQLMixinDallinger, dallinger.models.Participant):
             "accumulated_reward": "$" + "{:.2f}".format(self.calculate_reward()),
         }
 
+    def fail(self, reason=None):
+        if self.failed:
+            logger.info("Participant %i already failed, not failing again.", self.id)
+            return
+
+        if reason is not None:
+            self.append_failure_tags(reason)
+        reason = ", ".join(self.failure_tags)
+
+        logger.info(
+            "Failing participant %i (reason: %s)",
+            self.id,
+            reason,
+        )
+
+        from psynet.experiment import get_experiment
+
+        exp = get_experiment()
+
+        for i, routine in enumerate(exp.participant_fail_routines):
+            logger.info(
+                "Executing fail routine %i/%i ('%s')...",
+                i + 1,
+                len(exp.participant_fail_routines),
+                routine.label,
+            )
+            call_function_with_context(
+                routine.function,
+                participant=self,
+                experiment=self,
+            )
+
+        super().fail(reason=reason)
+        for group in self.active_sync_groups.values():
+            from .sync import SimpleSyncGroup
+
+            if isinstance(group, SimpleSyncGroup):
+                group.check_numbers()
+
 
 def get_participant(participant_id: int, for_update: bool = False) -> Participant:
     """
@@ -630,83 +731,3 @@ def get_participant(participant_id: int, for_update: bool = False) -> Participan
     if for_update:
         query = query.with_for_update(of=Participant).populate_existing()
     return query.one()
-
-
-class TimeCreditStore:
-    fields = [
-        "confirmed_credit",
-        "is_fixed",
-        "pending_credit",
-        "max_pending_credit",
-        "wage_per_hour",
-        "experiment_max_time_credit",
-        "experiment_max_reward",
-    ]
-
-    def __init__(self, participant):
-        self.participant = participant
-
-    def get_internal_name(self, name):
-        if name not in self.fields:
-            raise ValueError(f"{name} is not a valid field for TimeCreditStore.")
-        return f"__time_credit__{name}"
-
-    def __getattr__(self, name):
-        if name == "participant":
-            return self.__dict__["participant"]
-        else:
-            return self.participant.var.get(self.get_internal_name(name))
-
-    def __setattr__(self, name, value):
-        if name == "participant":
-            self.__dict__["participant"] = value
-        else:
-            self.participant.var.set(self.get_internal_name(name), value)
-
-    def initialize(self, experiment):
-        from .experiment import get_and_load_config
-
-        self.confirmed_credit = 0.0
-        self.is_fixed = False
-        self.pending_credit = 0.0
-        self.max_pending_credit = 0.0
-        self.wage_per_hour = get_and_load_config().get("wage_per_hour")
-
-        experiment_estimated_time_credit = experiment.timeline.estimated_time_credit
-        self.experiment_max_time_credit = experiment_estimated_time_credit.get_max(
-            "time"
-        )
-        self.experiment_max_reward = experiment_estimated_time_credit.get_max(
-            "reward", wage_per_hour=self.wage_per_hour
-        )
-
-    def increment(self, value: float):
-        if self.is_fixed:
-            self.pending_credit += value
-            if self.pending_credit > self.max_pending_credit:
-                self.pending_credit = self.max_pending_credit
-        else:
-            self.confirmed_credit += value
-
-    def start_fix_time(self, time_estimate: float):
-        assert not self.is_fixed
-        self.is_fixed = True
-        self.pending_credit = 0.0
-        self.max_pending_credit = time_estimate
-
-    def end_fix_time(self, time_estimate: float):
-        assert self.is_fixed
-        self.is_fixed = False
-        self.pending_credit = 0.0
-        self.max_pending_credit = 0.0
-        self.confirmed_credit += time_estimate
-
-    def get_time_reward(self):
-        return self.wage_per_hour * self.confirmed_credit / (60 * 60)
-
-    def estimate_time_credit(self):
-        return self.confirmed_credit + self.pending_credit
-
-    @property
-    def progress(self):
-        return self.estimate_time_credit() / self.experiment_max_time_credit
