@@ -18,7 +18,11 @@ from dallinger.bots import BotBase
 from dallinger.config import get_config
 from dallinger.models import Node
 from dallinger.pytest_dallinger import flush_output
-from selenium.common.exceptions import NoSuchElementException, TimeoutException
+from selenium.common.exceptions import (
+    ElementClickInterceptedException,
+    NoSuchElementException,
+    TimeoutException,
+)
 from selenium.webdriver.common.by import By
 from selenium.webdriver.support import expected_conditions as EC
 from selenium.webdriver.support.ui import WebDriverWait
@@ -31,14 +35,21 @@ from .command_line import (
 )
 from .data import init_db
 from .experiment import get_experiment, import_local_experiment
+from .modular_page import ModularPage, PushButtonControl
 from .redis import redis_vars
 from .trial.main import TrialNetwork
+from .trial.static import StaticNode, StaticTrial, StaticTrialMaker
 from .utils import clear_all_caches, wait_until
 
 logger = logging.getLogger(__file__)
 warnings.filterwarnings("ignore", category=sqlalchemy.exc.SAWarning)
-warnings.filterwarnings(
-    "ignore", "color, on_color and attrs are not supported when running in jupyter"
+
+ci_only = pytest.mark.skipif(
+    not os.environ.get("CI"), reason="This test only runs in CI environment"
+)
+
+local_only = pytest.mark.skipif(
+    os.environ.get("CI"), reason="This test only runs in local environment"
 )
 
 
@@ -86,12 +97,51 @@ def bot_class(headless=None):
             consent, and instruction pages.
             """
             try:
+                self.driver.set_window_size(1024, 768)
+
                 self.driver.get(self.URL)
                 logger.info("Loaded ad page.")
-                begin = WebDriverWait(self.driver, 10).until(
-                    EC.element_to_be_clickable((By.CSS_SELECTOR, "button.btn-primary"))
+
+                # First ensure the page is fully loaded
+                WebDriverWait(self.driver, 10).until(
+                    lambda d: page_loaded(d), message="Page failed to load completely"
                 )
-                begin.click()
+                logger.info("Page fully loaded.")
+
+                # Then check for the button
+                begin = WebDriverWait(self.driver, 10).until(
+                    EC.element_to_be_clickable((By.CSS_SELECTOR, "button.btn-primary")),
+                    message="Begin experiment button not found or not clickable",
+                )
+
+                # Scroll button into view and wait until it's actually visible
+                self.driver.execute_script(
+                    "arguments[0].scrollIntoView({block: 'center'});", begin
+                )
+
+                # Wait until the button is actually in view
+                WebDriverWait(self.driver, 10).until(
+                    lambda d: d.execute_script(
+                        """
+                        const rect = arguments[0].getBoundingClientRect();
+                        return (
+                            rect.top >= 0 &&
+                            rect.left >= 0 &&
+                            rect.bottom <= window.innerHeight &&
+                            rect.right <= window.innerWidth
+                        );
+                    """,
+                        begin,
+                    ),
+                    message="Button failed to scroll into view",
+                )
+
+                try:
+                    begin.click()
+                except ElementClickInterceptedException:
+                    _debug_click_interception(self.driver, begin)
+                    raise
+
                 logger.info("Clicked begin experiment button.")
 
                 experiment = get_experiment()
@@ -139,7 +189,10 @@ def bot_class(headless=None):
             if headless:
                 chrome_options.add_argument("--headless")
 
-            return webdriver.Chrome(options=chrome_options)
+            driver = webdriver.Chrome(options=chrome_options)
+            driver.set_window_size(1024, 768)
+
+            return driver
 
     return PYTEST_BOT_CLASS
 
@@ -180,7 +233,13 @@ def next_page(driver, button_identifier, by=By.ID, finished=False, max_wait=10.0
 
     old_uuid = get_uuid()
     click_button()
-    if not finished:
+    if finished:
+        wait_until(
+            lambda: "recruiter-exit" in driver.current_url,
+            max_wait=max_wait,
+            error_message="Never reached the recruiter-exit route, seems like the experiment never finished.",
+        )
+    else:
         if driver.current_url == "http://localhost:5000/error-page":
             raise RuntimeError(
                 "Unexpectedly hit an error page, check the server logs for details."
@@ -193,6 +252,10 @@ def next_page(driver, button_identifier, by=By.ID, finished=False, max_wait=10.0
         )
 
 
+def click_finish_button(driver):
+    next_page(driver, "Finish", finished=True)
+
+
 @pytest.fixture
 def deployment_info():
     from psynet import deployment_info
@@ -321,10 +384,17 @@ def debug_experiment(
     if not config.ready:
         config.load()
 
+    config.set("dashboard_user", "test_admin")
+    config.set("dashboard_password", "test_password")
+
     p = pexpect.spawn(
         "psynet",
         ["debug", "local", "--legacy", "--no-browsers"],
-        env=env,
+        env={
+            **env,
+            "dashboard_user": "test_admin",
+            "dashboard_password": "test_password",
+        },
         encoding="utf-8",
     )
     patch_pexpect_error_reporter(p)
@@ -482,5 +552,210 @@ def deployment_id():
 #         raise
 
 
-def path_to_demo(demo):
-    return Path(__file__).parent.parent.joinpath("demos").joinpath(demo).__str__()
+def path_to_demo_experiment(demo):
+    return (
+        Path(__file__)
+        .parent.parent.joinpath("demos/experiments")
+        .joinpath(demo)
+        .__str__()
+    )
+
+
+def path_to_demo_feature(demo):
+    return (
+        Path(__file__).parent.parent.joinpath("demos/features").joinpath(demo).__str__()
+    )
+
+
+def path_to_test_experiment(experiment):
+    return (
+        Path(__file__)
+        .parent.parent.joinpath("tests/experiments")
+        .joinpath(experiment)
+        .__str__()
+    )
+
+
+nodes_1 = [
+    StaticNode(
+        definition={"animal": animal},
+        block=block,
+    )
+    for animal in ["cats", "dogs", "fish", "ponies"]
+    for block in ["A", "B", "C"]
+]
+
+
+class AnimalTrial(StaticTrial):
+    """
+    A trial class for use in tests.
+    """
+
+    time_estimate = 3
+
+    def show_trial(self, experiment, participant):
+        animal = self.definition["animal"]
+        return ModularPage(
+            "animal_trial",
+            f"How much do you like {animal}?",
+            PushButtonControl(
+                ["Not at all", "A little", "Very much"],
+            ),
+            time_estimate=self.time_estimate,
+        )
+
+
+nodes_2 = [
+    StaticNode(
+        definition={"color": color},
+        block=block,
+    )
+    for color in ["red", "green", "blue", "orange"]
+    for block in ["A", "B", "C"]
+]
+
+
+class ColorTrial(StaticTrial):
+    """
+    A trial class for use in tests.
+    """
+
+    time_estimate = 3
+
+    def show_trial(self, experiment, participant):
+        color = self.definition["color"]
+        return ModularPage(
+            "color_trial",
+            f"How much do you like {color}?",
+            PushButtonControl(
+                ["Not at all", "A little", "Very much"],
+            ),
+            time_estimate=self.time_estimate,
+        )
+
+
+# A trial maker for use in tests
+trial_maker_1 = StaticTrialMaker(
+    id_="animals",
+    trial_class=AnimalTrial,
+    nodes=nodes_1,
+    expected_trials_per_participant=6,
+    max_trials_per_block=2,
+    allow_repeated_nodes=True,
+    balance_across_nodes=True,
+    check_performance_at_end=False,
+    check_performance_every_trial=False,
+    target_n_participants=1,
+    target_trials_per_node=None,
+    recruit_mode="n_participants",
+    n_repeat_trials=3,
+)
+
+# A trial maker for use in tests
+trial_maker_2 = StaticTrialMaker(
+    id_="colors",
+    trial_class=ColorTrial,
+    nodes=nodes_2,
+    expected_trials_per_participant=6,
+    max_trials_per_block=2,
+    allow_repeated_nodes=True,
+    balance_across_nodes=True,
+    check_performance_at_end=False,
+    check_performance_every_trial=False,
+    target_n_participants=1,
+    target_trials_per_node=None,
+    recruit_mode="n_participants",
+    n_repeat_trials=3,
+)
+
+
+def _debug_click_interception(driver, element):
+    """Debug what's intercepting a click on an element.
+
+    Parameters
+    ----------
+    driver : WebDriver
+        The Selenium WebDriver instance
+    element : WebElement
+        The element that failed to be clicked
+
+    Returns
+    -------
+    None
+        Logs debug information about what's intercepting the click
+    """
+    location = element.location
+    size = element.size
+    click_x = location["x"] + size["width"] // 2
+    click_y = location["y"] + size["height"] // 2
+
+    # Log viewport size and scroll position
+    viewport_size = driver.execute_script(
+        """
+        return {
+            width: window.innerWidth,
+            height: window.innerHeight,
+            scrollX: window.scrollX,
+            scrollY: window.scrollY
+        };
+    """
+    )
+    logger.error(f"Viewport size and scroll: {viewport_size}")
+
+    # Check if element is in viewport
+    is_in_viewport = driver.execute_script(
+        """
+        const rect = arguments[0].getBoundingClientRect();
+        return {
+            isVisible: rect.width > 0 && rect.height > 0,
+            isInViewport: (
+                rect.top >= 0 &&
+                rect.left >= 0 &&
+                rect.bottom <= window.innerHeight &&
+                rect.right <= window.innerWidth
+            ),
+            rect: {
+                top: rect.top,
+                left: rect.left,
+                bottom: rect.bottom,
+                right: rect.right
+            }
+        };
+    """,
+        element,
+    )
+    logger.error(f"Element viewport status: {is_in_viewport}")
+
+    # Get element at click coordinates
+    element_at_point = driver.execute_script(
+        """
+        return document.elementFromPoint(arguments[0], arguments[1]);
+        """,
+        click_x,
+        click_y,
+    )
+
+    # Get all elements that overlap with our button
+    overlapping_elements = driver.execute_script(
+        """
+        const button = arguments[0];
+        const buttonRect = button.getBoundingClientRect();
+        const elements = document.elementsFromPoint(
+            buttonRect.left + buttonRect.width/2,
+            buttonRect.top + buttonRect.height/2
+        );
+        return elements.map(el => ({
+            tag: el.tagName,
+            id: el.id,
+            class: el.className,
+            text: el.textContent?.trim()
+        }));
+        """,
+        element,
+    )
+
+    logger.error(f"Button location: {location}, size: {size}")
+    logger.error(
+        f"Element at click point: {element_at_point.get_attribute('outerHTML') if element_at_point else 'None'}"
+    )
+    logger.error(f"Overlapping elements: {overlapping_elements}")
