@@ -1,5 +1,5 @@
 import random
-from typing import Optional, Type
+from typing import List, Optional, Type, Union
 
 from dallinger import db
 from dallinger.models import Vector
@@ -22,8 +22,16 @@ from tqdm import tqdm
 from ..data import SQLMixinDallinger
 from ..field import PythonList, PythonObject, VarStore
 from ..page import wait_while
+from ..participant import Participant
+from ..sync import SyncGroup
 from ..timeline import is_list_of
-from ..utils import call_function_with_context, get_logger, log_time_taken, negate
+from ..utils import (
+    NoArgumentProvided,
+    call_function_with_context,
+    get_logger,
+    log_time_taken,
+    negate,
+)
 from .main import (
     NetworkTrialMaker,
     NetworkTrialMakerState,
@@ -99,6 +107,13 @@ class ChainNetwork(TrialNetwork):
         If ``participant is not None``, then this provides an optional ID for the network
         that is unique within a given participant.
 
+    sync_group_type : Optional[str]
+        The ``sync_group_type`` attribute of the trial maker that owns this network.
+
+    sync_group : Optional[SyncGroup]
+        The SyncGroup that owns this network (normally only relevant for within-style chains).
+
+
     Attributes
     ----------
 
@@ -146,6 +161,7 @@ class ChainNetwork(TrialNetwork):
     head_id = Column(Integer, ForeignKey("node.id"))
     trials_per_node = Column(Integer)
     definition = Column(PythonObject)
+    context = Column(PythonObject)
     block = Column(String, index=True)
 
     head = relationship(
@@ -162,8 +178,15 @@ class ChainNetwork(TrialNetwork):
         target_n_nodes: int,
         participant=None,
         id_within_participant: Optional[int] = None,
+        sync_group_type: Optional[str] = None,
+        sync_group: Optional[SyncGroup] = None,
     ):
-        super().__init__(trial_maker_id, experiment)
+        super().__init__(
+            trial_maker_id,
+            experiment,
+            sync_group_type=sync_group_type,
+            sync_group=sync_group,
+        )
         db.session.add(self)
 
         if participant is not None:
@@ -324,6 +347,8 @@ class ChainNetwork(TrialNetwork):
 
     def add_node(self, node):
         node.set_network(self)
+        if node.degree == 0:
+            self.context = node.context
         if node.degree > 0:
             # previous_head = self.get_node_with_degree(node.degree - 1)
             previous_head = self.head
@@ -499,6 +524,7 @@ class ChainNode(TrialNode):
     key = Column(String, index=True)
     degree = Column(Integer)
     target_n_trials = Column(Integer)
+    ready_to_spawn = Column(Boolean)
     child_id = Column(Integer, ForeignKey("node.id"), index=True)
     parent_id = Column(Integer, ForeignKey("node.id"), index=True)
     seed = Column(PythonObject, default=lambda: {})
@@ -523,6 +549,14 @@ class ChainNode(TrialNode):
         post_update=True,
     )
 
+    @property
+    def chain(self):
+        return self.network
+
+    @chain.setter
+    def chain(self, value):
+        self.network = value
+
     def __init__(
         self,
         *,
@@ -556,6 +590,9 @@ class ChainNode(TrialNode):
             else:
                 block = "default"
 
+        if not isinstance(block, str):
+            raise ValueError(f"block must be a string (got {block}).")
+
         if degree is None:
             if parent:
                 degree = parent.degree + 1
@@ -563,6 +600,7 @@ class ChainNode(TrialNode):
                 degree = 0
 
         self.degree = degree
+        self.ready_to_spawn = False
 
         if module_id is None:
             if parent:
@@ -572,21 +610,22 @@ class ChainNode(TrialNode):
             if parent:
                 context = parent.context
 
-        if not definition and not seed and degree == 0:
-            seed = self.create_initial_seed(experiment, participant)
-
-        if not definition:
-            definition = self.create_definition_from_seed(seed, experiment, participant)
-
         if assets is None:
             assets = {}
 
+        if not definition and not self.definition:
+            if not seed and degree == 0:
+                seed = self.create_initial_seed(experiment, participant)
+            definition = self.create_definition_from_seed(seed, experiment, participant)
+
+        if definition:
+            self.definition = definition
+
         self.assets = assets
         self.block = block
         self.participant_group = participant_group
         self.module_id = module_id
         self.seed = seed
-        self.definition = definition
         self.context = context
         self.propagate_failure = propagate_failure
         self._staged_assets = assets
@@ -713,19 +752,16 @@ class ChainNode(TrialNode):
 
     @hybrid_property
     def reached_target_n_trials(self):
-        return self.n_completed_and_processed_trials >= self.target_n_trials
+        if self.target_n_trials is None:
+            return False
+        else:
+            return self.n_completed_and_processed_trials >= self.target_n_trials
 
-    @reached_target_n_trials.expression
-    def reached_target_n_trials(cls):
-        return cls.n_completed_and_processed_trials >= cls.target_n_trials
+    def check_ready_to_spawn(self):
+        self.ready_to_spawn = self._ready_to_spawn()
 
-    @hybrid_property
-    def ready_to_spawn(self):
-        return self.reached_target_n_trials
-
-    @ready_to_spawn.expression
-    def ready_to_spawn(cls):
-        return cls.reached_target_n_trials
+    def _ready_to_spawn(self):
+        return self.reached_target_n_trials and len(self.pending_trials) == 0
 
     @property
     def viable_trials(self):
@@ -906,6 +942,14 @@ class ChainTrial(Trial):
     block_position = Column(Integer, index=True)
     block = Column(String, index=True)
 
+    @property
+    def chain(self):
+        return self.network
+
+    @chain.setter
+    def chain(self, value):
+        self.network = value
+
     def __init__(self, experiment, node, participant, *args, **kwargs):
         super().__init__(experiment, node, participant, *args, **kwargs)
         if participant.in_module and hasattr(
@@ -923,6 +967,11 @@ class ChainTrial(Trial):
     # def node(self):
     #     return self.origin
 
+    def fail(self, reason=None):
+        super().fail(reason)
+        if isinstance(self.node, ChainNode):
+            self.node.check_ready_to_spawn()
+
     @property
     def failure_cascade(self):
         to_fail = []
@@ -937,6 +986,7 @@ class ChainTrial(Trial):
 
     def on_finalized(self):
         super().on_finalized()
+        self.node.check_ready_to_spawn()
         if self.trial_maker and self.trial_maker.chain_type == "within":
             self.trial_maker.call_grow_network(network=self.network)
 
@@ -1035,6 +1085,10 @@ class ChainTrialMaker(NetworkTrialMaker):
         Either ``"within"`` for within-participant chains,
         or ``"across"`` for across-participant chains.
 
+    expected_trials_per_participant
+        Expected number of trials that each participant will complete.
+        This is used for timeline/progress estimation purposes.
+
     max_trials_per_participant
         Maximum number of trials that each participant may complete;
         once this number is reached, the participant will move on
@@ -1143,6 +1197,11 @@ class ChainTrialMaker(NetworkTrialMaker):
         of each SyncGroup. The other members of this SyncGroup will follow that leader around,
         so that in every given trial the SyncGroup works on the same node together.
 
+    sync_group_max_wait_time
+        The maximum time that the participant will be allowed to wait for the SyncGroup to be ready.
+        If this time is exceeded then the participant will be failed and the experiment will
+        terminate early. Defaults to 45.0 seconds.
+
 
     Attributes
     ----------
@@ -1197,7 +1256,7 @@ class ChainTrialMaker(NetworkTrialMaker):
         network_class: Type[ChainNetwork] = None,
         chain_type: str,
         expected_trials_per_participant: int,
-        max_trials_per_participant: Optional[int] = None,
+        max_trials_per_participant: Optional[int] = NoArgumentProvided,
         max_trials_per_block: Optional[int] = None,
         max_nodes_per_chain: Optional[int] = None,
         chains_per_participant: Optional[int] = None,
@@ -1206,7 +1265,7 @@ class ChainTrialMaker(NetworkTrialMaker):
         n_repeat_trials: int = 0,
         target_n_participants: Optional[int] = None,
         balance_across_chains: bool = False,
-        start_nodes=None,
+        start_nodes: Optional[Union[callable, List[ChainNode]]] = None,
         # balance_strategy: Set[str] = {"within", "across"},
         check_performance_at_end: bool = False,
         check_performance_every_trial: bool = False,
@@ -1219,7 +1278,11 @@ class ChainTrialMaker(NetworkTrialMaker):
         assets=None,
         choose_participant_group: Optional[callable] = None,
         sync_group_type: Optional[str] = None,
+        sync_group_max_wait_time: float = 45.0,
     ):
+        if max_trials_per_participant == NoArgumentProvided:
+            max_trials_per_participant = None
+
         if network_class is None:
             network_class = self.default_network_class
 
@@ -1315,6 +1378,7 @@ class ChainTrialMaker(NetworkTrialMaker):
             wait_for_networks=wait_for_networks,
             assets=assets,
             sync_group_type=sync_group_type,
+            sync_group_max_wait_time=sync_group_max_wait_time,
         )
 
         self.check_initialization()
@@ -1342,22 +1406,38 @@ class ChainTrialMaker(NetworkTrialMaker):
     def init_participant(self, experiment, participant):
         super().init_participant(experiment, participant)
         participant.module_state.participated_networks = []
-        if self.chain_type == "within":
-            networks = self.create_networks_within(experiment, participant)
+
+        sync_group = (
+            participant.active_sync_groups[self.sync_group_type]
+            if self.sync_group_type
+            else None
+        )
+        is_follower = sync_group and participant != sync_group.leader
+
+        if not is_follower:
+            if self.chain_type == "within":
+                networks = self.create_networks_within(experiment, participant)
+            else:
+                networks = self.networks
+                if len(self.networks) == 0:
+                    raise RuntimeError(
+                        f"Couldn't find any networks for the trial maker '{participant.module_state.module_id}'. "
+                        "A common reason for this is deploying your experiment using 'dallinger deploy' instead of "
+                        "'psynet deploy'. "
+                        "Another common reason is reloading the experiment in debug mode after adding a new trial maker. "
+                        "In the latter case you need to restart the debug session before continuing."
+                    )
+            self.check_participant_groups(networks)
+
+            blocks = set([network.block for network in networks])
+            self.init_block_order(experiment, participant, blocks)
         else:
-            networks = self.networks
-            if len(self.networks) == 0:
-                raise RuntimeError(
-                    f"Couldn't find any networks for the trial maker '{participant.module_state.module_id}'. "
-                    "A common reason for this is deploying your experiment using 'dallinger deploy' instead of "
-                    "'psynet deploy'. "
-                    "Another common reason is reloading the experiment in debug mode after adding a new trial maker. "
-                    "In the latter case you need to restart the debug session before continuing."
-                )
-        blocks = set([network.block for network in networks])
-        self.init_block_order(experiment, participant, blocks)
-        participant.module_state.set_block_position(0)
-        self.check_participant_groups(networks)
+            participant.module_state.block_order = (
+                sync_group.leader.module_state.block_order
+            )
+            participant.module_state.set_block_position(
+                sync_group.leader.module_state.block_position
+            )
 
     def init_block_order(self, experiment, participant, blocks):
         block_order = call_function_with_context(
@@ -1367,6 +1447,7 @@ class ChainTrialMaker(NetworkTrialMaker):
             blocks=blocks,
         )
         participant.module_state.block_order = block_order
+        participant.module_state.set_block_position(0)
 
     def choose_block_order(self, experiment, participant, blocks):
         # pylint: disable=unused-argument
@@ -1479,7 +1560,7 @@ class ChainTrialMaker(NetworkTrialMaker):
         if self.chain_type == "across":
             self.create_networks_across(experiment)
 
-    def create_networks_within(self, experiment, participant):
+    def create_networks_within(self, experiment, participant: Participant):
         if self.start_nodes:
             nodes = call_function_with_context(
                 self.start_nodes, experiment=experiment, participant=participant
@@ -1540,6 +1621,11 @@ class ChainTrialMaker(NetworkTrialMaker):
                 network=None, experiment=experiment, participant=participant
             )
 
+        if participant is not None and self.sync_group_type is not None:
+            sync_group = participant.active_sync_groups[self.sync_group_type]
+        else:
+            sync_group = None
+
         network = self.network_class(
             trial_maker_id=self.id,
             start_node=start_node,
@@ -1549,6 +1635,8 @@ class ChainTrialMaker(NetworkTrialMaker):
             target_n_nodes=self.max_nodes_per_chain,
             participant=participant,
             id_within_participant=id_within_participant,
+            sync_group_type=self.sync_group_type,
+            sync_group=sync_group,
         )
         db.session.add(network)
         start_node.set_network(network)
@@ -1594,7 +1682,12 @@ class ChainTrialMaker(NetworkTrialMaker):
             ):
                 return "exit"
             else:
-                participant.module_state.go_to_next_block()
+                if self.sync_group_type:
+                    group = participant.active_sync_groups[self.sync_group_type]
+                    for p in group.participants:
+                        p.module_state.go_to_next_block()
+                else:
+                    participant.module_state.go_to_next_block()
 
         # networks = db.session.query(
         #     self.network_class.chain_type,
@@ -1616,7 +1709,12 @@ class ChainTrialMaker(NetworkTrialMaker):
         # )
 
         if self.chain_type == "within":
-            networks = self.filter_by_participant_id(networks, participant)
+            if self.sync_group_type is not None:
+                sync_group = participant.active_sync_groups[self.sync_group_type]
+                assert sync_group.id is not None
+                networks = networks.filter_by(sync_group_id=sync_group.id)
+            else:
+                networks = self.filter_by_participant_id(networks, participant)
         elif (
             self.chain_type == "across"
             and not self.allow_revisiting_networks_in_across_chains
@@ -1635,7 +1733,8 @@ class ChainTrialMaker(NetworkTrialMaker):
         networks = networks.all()
 
         networks = self.custom_network_filter(
-            candidates=networks, participant=participant
+            candidates=networks,
+            participant=participant,
         )
 
         logger.info("%i remain after applying custom network filters.", len(networks))
@@ -1722,7 +1821,11 @@ class ChainTrialMaker(NetworkTrialMaker):
 
         networks = self.prioritize_networks(networks, participant, experiment)
 
+        if len(networks) == 0:
+            return "exit"
+
         chosen = networks[0]
+
         if chosen.block != current_block:
             logger.info(
                 f"Advanced from block '{current_block}' to '{chosen.block}' "
